# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from datetime import datetime
from datetime import timedelta
from datetime import UTC

from gajim.common import app
from gajim.common.i18n import _
from gajim.common.i18n import ngettext
from gajim.common.util.user_strings import get_uf_relative_time


class GetRelativeTimeTest(unittest.TestCase):
    """Tests for the get_uf_relative_time function."""

    def test_sub_1_minute(self):
        """Test timedelta less than 1 minute"""
        timenow = datetime(2023, 1, 2, 3, 4, 0, tzinfo=UTC)
        timestamp1 = timenow - timedelta(seconds=30)
        self.assertEqual(get_uf_relative_time(timestamp1, timenow), _("Just now"))

    def test_sub_15_minutes(self):
        """Test timedelta less than 15 minutes and more than 1 minute ago"""
        timenow = datetime(2023, 1, 2, 3, 4, 0, tzinfo=UTC)
        timestamp1 = timenow - timedelta(minutes=3)
        self.assertEqual(
            get_uf_relative_time(timestamp1, timenow),
            ngettext("%(number)s min ago", "%(number)s mins ago", 3) % {"number": 3},
        )

    def test_sub_15_minutes_next_day(self):
        """Test timedelta less than 15 minutes and it is the next day"""
        timenow = datetime(2023, 1, 1, 0, 5, 0, tzinfo=UTC)
        timestamp1 = timenow - timedelta(minutes=10)
        self.assertEqual(
            get_uf_relative_time(timestamp1, timenow),
            ngettext("%(number)s min ago", "%(number)s mins ago", 10) % {"number": 10},
        )

    def test_today(self):
        """Test today: same day and more than 15 minutes ago"""
        timenow = datetime(2023, 1, 2, 12, 0, 0, tzinfo=UTC)
        timestamp1 = timenow - timedelta(hours=4)
        self.assertEqual(
            get_uf_relative_time(timestamp1, timenow),
            timestamp1.astimezone().strftime(app.settings.get("time_format")),
        )

    def test_yesterday_less_than_24h(self):
        """Test yesterday and less than 24h ago"""
        timenow = datetime(2023, 1, 2, 12, 0, 0, tzinfo=UTC)
        timestamp1 = datetime(2023, 1, 1, 14, 0, 0, tzinfo=UTC)
        self.assertEqual(get_uf_relative_time(timestamp1, timenow), _("Yesterday"))

    def test_yesterday_more_than_24h(self):
        """Test yesterday and more than 24h ago"""
        timenow = datetime(2023, 1, 2, 12, 0, 0, tzinfo=UTC)
        timestamp1 = datetime(2023, 1, 1, 10, 0, 0, tzinfo=UTC)
        self.assertEqual(get_uf_relative_time(timestamp1, timenow), _("Yesterday"))

    def test_weekday(self):
        """Test weekday: timestamp older than yesterday and less
        than 7 days ago, should return the weekday, i.e. 'Sun' for Sunday"""
        timenow = datetime(2023, 1, 5, 1, 2, 3, tzinfo=UTC)
        timestamp1 = datetime(2023, 1, 1, 4, 5, 6, tzinfo=UTC)
        self.assertEqual(
            get_uf_relative_time(timestamp1, timenow),
            timestamp1.astimezone().strftime("%a"),
        )

    def test_month_day(self):
        """Test month_day: timestamp more than 7 days ago but less than 365"""
        timenow = datetime(2023, 1, 5, 1, 2, 3, tzinfo=UTC)
        timestamp1 = datetime(2022, 11, 15, 4, 5, 6, tzinfo=UTC)
        self.assertEqual(
            get_uf_relative_time(timestamp1, timenow),
            timestamp1.astimezone().strftime("%b %d"),
        )

    def test_year(self):
        """Test year: timestamp more than 365 days ago"""
        timenow = datetime(2023, 1, 5, 1, 2, 3, tzinfo=UTC)
        timestamp1 = datetime(2022, 1, 1, 4, 5, 6, tzinfo=UTC)
        self.assertEqual(get_uf_relative_time(timestamp1, timenow), "2022")


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

from unittest.mock import MagicMock

from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.call_manager import CallManager
from gajim.common.const import AvatarSize
from gajim.common.modules.contacts import BareContact
from gajim.common.modules.contacts import ResourceContact

from gajim.gtk.avatar import convert_surface_to_texture
from gajim.gtk.avatar import generate_default_avatar
from gajim.gtk.call_window import CallWindow

from . import util

ACCOUNT = "testacc1"
BARE_JID = JID.from_string("contact@test.tld")
RESOURCE_JID = JID.from_string("contact@test.tld/test")


class TestContactsModule:
    def __init__(self) -> None:
        pass

    @staticmethod
    def get_contact(jid: str | JID) -> BareContact | ResourceContact:
        if jid == RESOURCE_JID:
            contact = MagicMock(spec_set=ResourceContact)
            contact.jid = RESOURCE_JID
        else:
            contact = MagicMock(spec_set=BareContact)
            contact.jid = BARE_JID
            contact.name = "Test Contact"
            avatar = convert_surface_to_texture(
                generate_default_avatar("T", (0.2, 0.1, 0.7), AvatarSize.CALL_BIG, 1)
            )
            contact.get_avatar = MagicMock(return_value=avatar)

        contact.account = ACCOUNT
        contact.is_groupchat = False

        return contact


class TestClient:
    def __init__(self, account: str) -> None:
        self.account = account

    def get_module(self, module: str) -> Any:
        if module == "Contacts":
            return TestContactsModule()
        return MagicMock()


util.init_settings()

app.get_client = MagicMock(side_effect=TestClient)
app.call_manager = CallManager()

window = CallWindow(ACCOUNT, RESOURCE_JID)

# Enable popover buttons for debugging
window._ui.audio_buttons_box.set_sensitive(True)  # pyright: ignore
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gajim.common.util.uri import DataUri
from gajim.common.util.uri import XmppIri


class TestUriUtil(unittest.TestCase):
    def test_parsing_xmpp_iri(self) -> None:
        xiri = XmppIri.from_string(
            r"xmpp:nasty!%23$%25()*+,-.;=%3F%5B%5C%5D%5E_%60%7B%7C%7D~n@example.com"
        )
        self.assertEqual(str(xiri.jid), r"nasty!#$%()*+,-.;=?[\]^_`{|}~n@example.com")

        xiri = XmppIri.from_string(
            "xmpp:example-node@example.com?message;subject=Hello%20World"
        )
        self.assertEqual(str(xiri.jid), "example-node@example.com")
        self.assertEqual(xiri.action, "message")
        self.assertEqual(xiri.params, {"subject": "Hello World"})

        xiri = XmppIri.from_string(r"xmpp:aD%C3%BCrst@asd.at")
        self.assertEqual(str(xiri.jid), "adürst@asd.at")

    def test_parse_data_uri(self) -> None:
        test_uris = {
            "data:,A%20brief%20note": DataUri(
                scheme="data",
                uri="data:,A%20brief%20note",
                media_type="text/plain",
                parameters={},
                is_base64=False,
                data="A%20brief%20note",
            ),
            "data:image/gif;base64,R0lGODdhMAAwAPAAAAAAAP///ywA": DataUri(
                scheme="data",
                uri="data:image/gif;base64,R0lGODdhMAAwAPAAAAAAAP///ywA",
                media_type="image/gif",
                parameters={},
                is_base64=True,
                data="R0lGODdhMAAwAPAAAAAAAP///ywA",
            ),
            "data:text/plain;charset=iso-8859-7,%be%fg%be": DataUri(
                scheme="data",
                uri="data:text/plain;charset=iso-8859-7,%be%fg%be",
                media_type="text/plain",
                parameters={"charset": "iso-8859-7"},
                is_base64=False,
                data="%be%fg%be",
            ),
        }

        for uri, expected_res in test_uris.items():
            self.assertEqual(DataUri.from_string(uri), expected_res)


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

from pathlib import Path
from unittest.mock import MagicMock

from gi.repository import Adw
from gi.repository import GLib
from gi.repository import Gtk

from gajim.common import app
from gajim.common import configpaths
from gajim.common.helpers import Observable
from gajim.plugins.events import PluginRemoved
from gajim.plugins.manifest import PluginManifest

from gajim.gtk.plugins import Plugins

from . import util

Adw.init()

plugin_a_data = {
    "name": "Plugin A",
    "short_name": "plugin_a",
    "description": "Plugin A description",
    "authors": ["Author 1"],
    "homepage": "example.org",
    "config_dialog": False,
    "version": "1.0.0",
    "requirements": [],
    "platforms": [],
}
plugin_a_data_update = {
    "name": "Plugin A",
    "short_name": "plugin_a",
    "description": "Plugin A description",
    "authors": ["Author 1", "Author 2"],
    "homepage": "example.org/plugin",
    "config_dialog": False,
    "version": "1.0.1",
    "requirements": [],
    "platforms": [],
}
plugin_b_data = {
    "name": "Plugin B",
    "short_name": "plugin_b",
    "description": "Plugin B description",
    "authors": ["author"],
    "homepage": "example.org",
    "config_dialog": False,
    "version": "1.0.0",
    "requirements": [],
    "platforms": [],
}
plugin_c_data = {
    "name": "Plugin C",
    "short_name": "plugin_c",
    "description": "Plugin C description",
    "authors": ["author"],
    "homepage": "example.org",
    "config_dialog": False,
    "version": "1.0.0",
    "requirements": [],
    "platforms": [],
}


class GajimPluginTest:
    def __init__(self, manifest: PluginManifest) -> None:
        self._manifest = manifest

    @property
    def active(self) -> bool:
        return True

    @property
    def activatable(self) -> bool:
        return True

    @property
    def manifest(self) -> PluginManifest:
        return self._manifest

    @property
    def __path__(self) -> Path:
        return Path("")


class PluginManagerTest:
    def __init__(self) -> None:
        self._plugins: list[GajimPluginTest] = [
            GajimPluginTest(PluginManifest.from_manifest_json(plugin_a_data))
        ]

    @property
    def plugins(self) -> list[GajimPluginTest]:
        return self._plugins

    def get_plugin(self, short_name: str) -> GajimPluginTest | None:
        for plugin in self.plugins:
            if plugin.manifest.short_name == short_name:
                return plugin
        return None

    def deactivate_plugin(self, plugin: GajimPluginTest) -> None:
        pass

    def update_plugins(
        self,
        replace: bool = True,
        activate: bool = False,
        plugin_name: str | None = None,
    ) -> list[str]:
        return ["asd"]

    def mock_update_plugins(self, manifests: list[PluginManifest]) -> None:
        for manifest in manifests:
            for plugin in self._plugins:
                if plugin.manifest.short_name == manifest.short_name:
                    self._plugins.remove(plugin)

            self._plugins.append(GajimPluginTest(manifest=manifest))

    def uninstall_plugin(self, plugin: GajimPluginTest) -> None:
        self.plugins.remove(plugin)
        app.ged.raise_event(PluginRemoved(manifest=plugin.manifest))


class PluginRepositoryTest(Observable):
    def __init__(self) -> None:
        Observable.__init__(self)

        self._manifests: list[PluginManifest] = [
            PluginManifest.from_manifest_json(plugin_a_data),
            PluginManifest.from_manifest_json(plugin_b_data),
            PluginManifest.from_manifest_json(plugin_c_data),
        ]

    def get_manifests(self) -> list[PluginManifest]:
        return self._manifests

    def contains(self, short_name: str) -> bool:
        return True

    def download_plugins(self, manifests: list[PluginManifest]) -> None:
        self.notify("download-started", manifests)
        # Notify with repository manifests
        finished_updates: list[PluginManifest] = []
        for manifest in manifests:
            for repo_manifest in self._manifests:
                if repo_manifest.short_name == manifest.short_name:
                    finished_updates.append(repo_manifest)

        app.plugin_manager.mock_update_plugins(finished_updates)  # type: ignore

        GLib.timeout_add_seconds(1, self._notify_finished, finished_updates)

    def _notify_finished(self, manifests: list[PluginManifest]) -> None:
        for manifest in manifests:
            self.notify("download-finished", manifest)

    def trigger_updates_available(self) -> None:
        self._manifests = [
            PluginManifest.from_manifest_json(plugin_a_data_update),
            PluginManifest.from_manifest_json(plugin_b_data),
            PluginManifest.from_manifest_json(plugin_c_data),
        ]
        self.notify("plugin-updates-available", [self._manifests[0]])


util.init_settings()

app.plugin_manager = PluginManagerTest()
app.plugin_repository = PluginRepositoryTest()


def _on_trigger_update_clicked(_button: Gtk.Button) -> None:
    app.plugin_repository.trigger_updates_available()  # type: ignore


configpaths.get = MagicMock(return_value=Path())


update_button = Gtk.Button(label="Trigger Update")
update_button.connect("clicked", _on_trigger_update_clicked)
controls_box = Gtk.Box(spacing=12)
controls_box.append(update_button)

control_group = Adw.PreferencesGroup()
control_group.add(controls_box)

page = Adw.PreferencesPage()
page.add(Plugins())
page.add(control_group)

window = Adw.PreferencesDialog()
window.add(page)
window.present()

util.run_app()



"""
This type stub file was generated by pyright.
"""

import platform

from ._version import __author__
from ._version import __description__
from ._version import __license__
from ._version import __title__
from ._version import __url__
from ._version import __version__
from .events import ToastActivatedEventArgs
from .events import ToastDismissalReason
from .events import ToastDismissedEventArgs
from .events import ToastFailedEventArgs
from .exceptions import InvalidImageException
from .exceptions import ToastNotFoundError
from .exceptions import UnsupportedOSVersionException
from .toast import Toast
from .toast_audio import AudioSource
from .toast_audio import ToastAudio
from .toasters import InteractableWindowsToaster
from .toasters import WindowsToaster
from .wrappers import ToastButton
from .wrappers import ToastButtonColour
from .wrappers import ToastDisplayImage
from .wrappers import ToastDuration
from .wrappers import ToastImage
from .wrappers import ToastImagePosition
from .wrappers import ToastInputSelectionBox
from .wrappers import ToastInputTextBox
from .wrappers import ToastProgressBar
from .wrappers import ToastScenario
from .wrappers import ToastSelection
from .wrappers import ToastSystemButton
from .wrappers import ToastSystemButtonAction

MIN_VERSION = ...
osVersion = int(platform.version().split(".")[2])
if platform.system() == "Windows" and osVersion < MIN_VERSION: ...
__all__ = [
    "__author__",
    "__description__",
    "__license__",
    "__title__",
    "__url__",
    "__version__",
    "ToastActivatedEventArgs",
    "ToastDismissalReason",
    "ToastDismissedEventArgs",
    "ToastFailedEventArgs",
    "InvalidImageException",
    "ToastNotFoundError",
    "UnsupportedOSVersionException",
    "AudioSource",
    "ToastAudio",
    "Toast",
    "InteractableWindowsToaster",
    "WindowsToaster",
    "ToastButton",
    "ToastButtonColour",
    "ToastDisplayImage",
    "ToastDuration",
    "ToastImage",
    "ToastImagePosition",
    "ToastInputSelectionBox",
    "ToastInputTextBox",
    "ToastProgressBar",
    "ToastScenario",
    "ToastSelection",
    "ToastSystemButton",
    "ToastSystemButtonAction",
]



from __future__ import annotations

from typing import Any

class CSSRule:
    STYLE_RULE: int

class CSSStyleSheet:
    cssText: bytes

    def __iter__(self) -> CSSStyleRule: ...
    def add(self, rule: CSSStyleRule) -> None: ...

class CSSStyleRule(CSSRule):
    style: CSSStyleDeclaration
    type: int
    selectorText: str

    def __init__(self, selectorText: str) -> None: ...
    def __next__(self) -> CSSStyleRule: ...

class CSSStyleDeclaration:
    def __getitem__(self, cssname: str) -> str: ...
    def __setitem__(self, cssname: str, value: Any) -> None: ...
    def getPropertyValue(self, property: str) -> Any: ...
    def removeProperty(self, property: str) -> None: ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from datetime import datetime
from datetime import UTC

from nbxmpp.protocol import JID
from sqlalchemy import select

from gajim.common import app
from gajim.common.helpers import get_uuid
from gajim.common.settings import Settings
from gajim.common.storage.archive.const import ChatDirection
from gajim.common.storage.archive.const import MessageState
from gajim.common.storage.archive.const import MessageType
from gajim.common.storage.archive.models import Message
from gajim.common.storage.archive.models import Moderation
from gajim.common.storage.archive.storage import MessageArchiveStorage


class ModerationTest(unittest.TestCase):
    def setUp(self) -> None:
        self._archive = MessageArchiveStorage(in_memory=True)
        self._archive.init()

        self._account = "testacc1"
        self._account_jid = JID.from_string("user@domain.org")
        self._remote_jid = JID.from_string("remote@jid.org")
        self._occupant_id = "occupantid1"
        self._init_settings()

    def tearDown(self) -> None:
        self._archive.shutdown()

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_insert_moderation(self) -> None:
        uuid = get_uuid()
        mod_data = Moderation(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            stanza_id=uuid,
            by=JID.from_string("some@domain.com"),
            reason="some reason",
            timestamp=datetime.fromtimestamp(0, UTC),
        )

        pk = self._archive.insert_row(mod_data, ignore_on_conflict=True)
        self.assertNotEqual(pk, -1)

        with self._archive.get_session() as s:
            moderation = s.scalar(select(Moderation).where(Moderation.pk == pk))
        assert moderation is not None

        self.assertEqual(moderation.by, "some@domain.com")
        self.assertIsInstance(moderation.by, JID)
        self.assertEqual(moderation.reason, "some reason")
        self.assertEqual(moderation.timestamp, datetime.fromtimestamp(0, UTC))
        self.assertEqual(moderation.stanza_id, uuid)
        self.assertEqual(moderation.occupant, None)

        mod_data = Moderation(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            stanza_id=uuid,
            by=JID.from_string("someother@domain.com"),
            reason="some other reason",
            timestamp=datetime.fromtimestamp(1, UTC),
        )

        pk = self._archive.insert_row(mod_data, ignore_on_conflict=True)
        self.assertEqual(pk, -1)

    def test_moderation_join(self) -> None:
        uuid = get_uuid()
        message_data = Message(
            account_=self._account,
            remote_jid_=self._remote_jid,
            resource="someres1",
            type=MessageType.CHAT,
            direction=ChatDirection.INCOMING,
            timestamp=datetime.now(UTC),
            state=MessageState.ACKNOWLEDGED,
            id="1",
            stanza_id=uuid,
            text="message",
        )

        pk = self._archive.insert_object(message_data)

        mod_data = Moderation(
            account_=self._account,
            remote_jid_=self._remote_jid,
            occupant_=None,
            stanza_id=uuid,
            by=JID.from_string("some@domain.com"),
            reason="some reason",
            timestamp=datetime.fromtimestamp(0, UTC),
        )

        self._archive.insert_row(mod_data, ignore_on_conflict=True)

        message = self._archive.get_message_with_pk(pk)

        assert message is not None
        assert message.moderation is not None
        self.assertEqual(message.moderation.by, "some@domain.com")
        self.assertIsInstance(message.moderation.by, JID)
        self.assertEqual(message.moderation.reason, "some reason")
        self.assertEqual(message.moderation.timestamp, datetime.fromtimestamp(0, UTC))
        self.assertEqual(message.moderation.stanza_id, uuid)
        self.assertEqual(message.moderation.occupant, None)


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Any

from unittest.mock import MagicMock

from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.events import PasswordRequired

from gajim.gtk.password_dialog import PasswordDialog

from . import util

ACCOUNT = "test"
FROM_JID = "contact@example.org"

util.init_settings()
app.settings.add_account(ACCOUNT)
app.settings.set_account_setting(ACCOUNT, "account_label", "Test")


def on_password(*args: Any) -> None:
    pass


class TestClient:
    def __init__(self, account: str) -> None:
        self.account = account

    def get_own_jid(self) -> JID:
        return JID.from_string(FROM_JID)


app.get_client = MagicMock(side_effect=TestClient)
app.get_account_label = MagicMock(return_value="Test Account")

client = app.get_client(ACCOUNT)

event = PasswordRequired(client, on_password)

window = PasswordDialog(ACCOUNT, event)
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

import unittest
from pathlib import Path

import sqlalchemy as sa

from gajim.common import app
from gajim.common.settings import Settings
from gajim.common.storage.archive import migration
from gajim.common.storage.archive.storage import CURRENT_USER_VERSION
from gajim.common.storage.archive.storage import MessageArchiveStorage


class TestMigration(unittest.TestCase):
    def setUp(self) -> None:
        self._init_settings()

    def tearDown(self) -> None:
        Path("test.db").unlink(missing_ok=True)
        Path("test.db-shm").unlink(missing_ok=True)
        Path("test.db-wal").unlink(missing_ok=True)

    def _init_settings(self) -> None:
        app.settings = Settings(in_memory=True)
        app.settings.init()
        app.settings.add_account("testacc1")
        app.settings.set_account_setting("testacc1", "address", "user@domain.org")

    def test_migration(self) -> None:
        # Database schema for Gajim 0.16.9
        DATABASE_SCHEMA = """
            CREATE TABLE jids(
                    jid_id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,
                    jid TEXT UNIQUE,
                    type INTEGER
            );

            CREATE TABLE unread_messages(
                    message_id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,
                    jid_id INTEGER,
                    shown BOOLEAN default 0
            );

            CREATE INDEX idx_unread_messages_jid_id ON unread_messages (jid_id);

            CREATE TABLE logs(
                    log_line_id INTEGER PRIMARY KEY AUTOINCREMENT UNIQUE,
                    jid_id INTEGER,
                    contact_name TEXT,
                    time INTEGER,
                    kind INTEGER,
                    show INTEGER,
                    message TEXT,
                    subject TEXT
            );

            CREATE INDEX idx_logs_jid_id_time ON logs (jid_id, time DESC);"""

        dbpath = Path("test.db")
        engine = sa.create_engine(f"sqlite:///{dbpath}", echo=False)

        with engine.connect() as connection:
            for stmt in DATABASE_SCHEMA.split(";"):
                connection.execute(sa.text(stmt))
                connection.commit()

        engine.dispose()

        archive = MessageArchiveStorage(path=dbpath)
        migration.run(archive, 0)

        version = archive.get_user_version()
        self.assertEqual(version, CURRENT_USER_VERSION)

        archive.shutdown()


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gajim.gtk.dataform import FakeDataFormWidget
from gajim.gtk.window import GajimAppWindow

from . import util

fake_form = {
    "instructions": ("This is the a long long long long long long test instruction"),
    "username": "",
    "nick": "",
    "password": "",
    "name": "",
    "first": "",
    "last": "",
    "email": "",
    "address": "",
    "city": "",
    "state": "",
    "zip": "",
    "phone": "",
    "url": "",
    "date": "",
    "misc": "",
    "text": "",
    "key": "",
}

fake_form2 = {
    "instructions": "To register, visit https://jabber.at/account/register/",
    "redirect-url": "https://jabber.at/account/register/",
}


class TestDataFormWindow(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        self._widget = FakeDataFormWidget(fake_form2)
        self.set_child(self._widget)

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestDataFormWindow()
window.show()

util.run_app()



# Copyright 2016 Christoph Reiter
#
# SPDX-License-Identifier: GPL-2.0-or-later

# Deletes unneeded DLLs and checks DLL dependencies.

from typing import cast

import logging
import os
import subprocess
import sys
from functools import cache
from multiprocessing import Process
from multiprocessing import Queue

import gi

gi.require_version("GIRepository", "3.0")
from gi.repository import GIRepository  # noqa: E402
from gi.repository import GLib  # noqa: E402

logging.basicConfig(level="INFO", format="%(levelname)s: %(message)s")
log = logging.getLogger()

IGNORED_LIBS = [("Soup", "2.4"), ("Gtk", "3.0")]

LibrariesDataT = tuple[list[str] | None, Exception | None]


def _get_shared_libraries(
    q: "Queue[LibrariesDataT]", namespace: str, version: str
) -> None:
    try:
        repo = GIRepository.Repository()
        repo.require(namespace, version, 0)  # type: ignore
        libs = cast(list[str], repo.get_shared_libraries(namespace))  # type: ignore
    except Exception as e:
        q.put((None, e))
    else:
        q.put((libs, None))


@cache
def get_shared_libraries(namespace: str, version: str) -> list[str] | None:
    # we have to start a new process because multiple versions can't be loaded
    # in the same process
    q: "Queue[LibrariesDataT]" = Queue()
    p = Process(target=_get_shared_libraries, args=(q, namespace, version))
    p.start()
    libs, error = q.get()
    p.join()
    if error is not None:
        raise error
    return libs


def get_required_by_typelibs() -> set[tuple[str, str, str]]:
    deps: set[tuple[str, str, str]] = set()
    repo = GIRepository.Repository()
    for tl in os.listdir(repo.get_search_path()[0]):
        namespace, version = os.path.splitext(tl)[0].split("-", 1)
        if (namespace, version) in IGNORED_LIBS:
            continue

        try:
            libs = get_shared_libraries(namespace, version)
        except GLib.Error as e:
            # g-i fails to load itself with a different version
            if "GIRepository" in e.message and "2.0" in e.message:
                continue
            else:
                raise

        if libs is None:
            continue

        for lib in libs:
            deps.add((namespace, version, lib))
    return deps


@cache
def get_dependencies(filename: str) -> list[str]:
    deps: list[str] = []
    try:
        data = subprocess.check_output(
            ["objdump", "-p", filename], stderr=subprocess.STDOUT
        )
    except subprocess.CalledProcessError:
        # can happen with wrong arch binaries
        return []
    data = data.decode("utf-8")
    for line in data.splitlines():
        line = line.strip()
        if line.startswith("DLL Name:"):
            deps.append(line.split(":", 1)[-1].strip().lower())
    return deps


def find_lib(root: str, name: str) -> bool:
    system_search_path = os.path.join("C:", os.sep, "Windows", "System32")
    if get_lib_path(root, name):
        return True
    elif os.path.exists(os.path.join(system_search_path, name)):
        return True
    elif name == "gdiplus.dll":
        return True
    elif name.startswith("msvcr"):
        return True
    elif name.startswith("api-ms-win-"):
        return True
    return False


def get_lib_path(root: str, name: str) -> str | None:
    search_path = os.path.join(root, "bin")
    if os.path.exists(os.path.join(search_path, name)):
        return os.path.join(search_path, name)


def get_things_to_delete(root: str) -> list[str]:
    extensions = [".exe", ".pyd", ".dll"]

    all_libs: set[str] = set()
    needed: set[str] = set()
    for base, _, files in os.walk(root):
        for f in files:
            lib = f.lower()
            path = os.path.join(base, f)
            ext_lower = os.path.splitext(f)[-1].lower()
            if ext_lower in extensions:
                if ext_lower == ".exe":
                    # we use .exe as dependency root
                    needed.add(lib)
                all_libs.add(f.lower())
                for lib in get_dependencies(path):
                    all_libs.add(lib)
                    needed.add(lib)
                    if not find_lib(root, lib):
                        print("MISSING:", path, lib)

    for namespace, version, lib in get_required_by_typelibs():
        all_libs.add(lib)
        needed.add(lib)
        if not find_lib(root, lib):
            print("MISSING:", namespace, version, lib)

    to_delete: list[str] = []
    for not_depended_on in all_libs - needed:
        path = get_lib_path(root, not_depended_on)
        if path:
            to_delete.append(path)

    return to_delete


def main() -> None:
    libs = get_things_to_delete(sys.prefix)
    while libs:
        for lib in libs:
            log.info("DELETE: %s", lib)
            os.unlink(lib)
        libs = get_things_to_delete(sys.prefix)


if __name__ == "__main__":
    main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gajim.common.open_graph_parser import OpenGraphData
from gajim.common.open_graph_parser import OpenGraphParser

EXAMPLE_1 = """
<!DOCTYPE html>
<html lang="en" dir="ltr">
<head>
  <meta charset="UTF-8">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<title>Gajim</title>
<meta property="og:title" content="Gajim" />
<meta property="og:description" content="Free and fully featured chat app for XMPP" />
<meta property="og:type" content="website" />
<meta property="og:url" content="https://gajim.org/" />
<meta property="og:image" content="https://gajim.org/img/og-image.png" />

  <link rel="alternate" type="application/rss+xml" href="https://gajim.org/index.xml" title="Gajim">


<link rel="me" href="https://fosstodon.org/@gajim">

<link rel="icon" type="image/png" href="/favicon-96x96.png" sizes="96x96" />
<link rel="icon" type="image/svg+xml" href="/favicon.svg" />
<link rel="shortcut icon" href="/favicon.ico" />
<link rel="apple-touch-icon" sizes="180x180" href="/apple-touch-icon.png" />
<meta name="apple-mobile-web-app-title" content="Gajim" />
<link rel="manifest" href="/site.webmanifest" />
<link rel="stylesheet" href="/font/inter/inter.css">
<link rel="stylesheet" href="/css/main.min.50abd93d84368b4faefaa11c38fabb177e442adde5a79989b65d96d7871bb4e9.css" integrity="sha256-UKvZPYQ2i0&#43;u&#43;qEcOPq7F35EKt3lp5mJtl2W14cbtOk=" crossorigin="anonymous">
<script src="/js/main.53216da438f76f46153f227656c218046669a230e3c456a5ec9fc06aaba51cf3.js" integrity="sha256-UyFtpDj3b0YVPyJ2VsIYBGZpojDjxFal7J/AaqulHPM=" crossorigin="anonymous"></script>
</head>
"""  # noqa: E501

EXAMPLE_2 = """
<title>title</title>
<meta name="description" content="description" />
<meta property="og:title" content="      

og title     " />
<meta property="og:description" content="og description" />
"""  # noqa: W291

EXAMPLE_3 = """
<title>title</title>
<meta name="description" content="description" />
"""


class OpenGraphParserTest(unittest.TestCase):
    """Tests parsing HTML for Open Graph meta data."""

    def test_parsing(self) -> None:
        parser = OpenGraphParser()
        result = parser.parse(EXAMPLE_1)

        expected_result = OpenGraphData(
            title="Gajim",
            description="Free and fully featured chat app for XMPP",
            image="https://gajim.org/img/og-image.png",
        )

        self.assertEqual(result, expected_result)

        parser = OpenGraphParser()
        result = parser.parse(EXAMPLE_2)

        expected_result = OpenGraphData(
            title="og title",
            description="og description",
        )

        self.assertEqual(result, expected_result)

        parser = OpenGraphParser()
        result = parser.parse(EXAMPLE_3)

        expected_result = OpenGraphData(
            title="title",
            description="description",
        )

        self.assertEqual(result, expected_result)


if __name__ == "__main__":
    unittest.main()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.protocol import JID

from gajim.common import app
from gajim.common.modules.contacts import GroupchatContact

from gajim.gtk.groupchat_state import GroupchatState
from gajim.gtk.window import GajimAppWindow

from . import util

ACCOUNT = "testacc1"
FROM_JID = "contact@test.tld"


class TestGroupchatState(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        self._box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            orientation=Gtk.Orientation.VERTICAL,
            spacing=12,
        )
        self.set_child(self._box)

        groupchat_state_widget = GroupchatState()
        self._box.append(groupchat_state_widget)

        contact = self._get_contact()
        groupchat_state_widget.switch_contact(contact)

    def _get_contact(self) -> GroupchatContact:
        contact = MagicMock(spec_set=GroupchatContact)
        contact.connect = MagicMock()
        contact.account = ACCOUNT
        contact.jid = JID.from_string(FROM_JID)
        contact.name = "Test Contact"
        contact.is_groupchat = True
        contact.is_joining = True
        contact.is_joined = False
        return contact

    def _cleanup(self) -> None:
        pass


app.get_client = MagicMock()

util.init_settings()

window = TestGroupchatState()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gdk
from gi.repository import Gtk

from gajim.gtk.window import GajimAppWindow

from . import util


class TestDNDFile(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            width_request=300,
            height_request=300,
        )
        box.add_css_class("dnd-area")
        self.set_child(box)

        drop_target = Gtk.DropTarget.new(Gdk.FileList, Gdk.DragAction.COPY)
        drop_target.connect("accept", self._on_drop_accept)
        drop_target.connect("drop", self._on_file_drop)
        box.add_controller(drop_target)

        label = Gtk.Label(label="Drop files here", hexpand=True)
        box.append(label)

    def _on_drop_accept(self, _target: Gtk.DropTarget, drop: Gdk.Drop) -> bool:
        formats = drop.get_formats()
        return bool(formats.contain_gtype(Gdk.FileList))

    def _on_file_drop(
        self, _target: Gtk.DropTarget, value: Gdk.FileList, _x: float, _y: float
    ) -> bool:
        files = value.get_files()
        print([file.get_basename() for file in files])
        return True

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestDNDFile()
window.show()

util.run_app()



"""
This type stub file was generated by pyright.
"""

import typing

import Foundation
import objc

from . import _base

class Icon(_base.Icon):
    _ACTION_SELECTOR: typing.Any = ...
    _MENU_ITEM_SELECTOR: typing.Any = ...
    HAS_DEFAULT_ACTION: bool = ...
    HAS_MENU_RADIO: bool = ...
    def __init__(self, *args: typing.Any, **kwargs: typing.Any) -> None: ...

class IconDelegate(Foundation.NSObject):
    @objc.namedSelector(Icon._ACTION_SELECTOR)
    def activate_button(self, sender: typing.Any) -> None: ...
    @objc.namedSelector(Icon._MENU_ITEM_SELECTOR)
    def activate_menu_item(self, sender: typing.Any) -> None: ...



"""
This type stub file was generated by pyright.
"""

DESTINATION: str = ...
PATH: str = ...

class Notifier:
    def __init__(self) -> None: ...
    def notify(self, title: str, message: str, icon: str) -> None:
        """Displays a notification message.

        :param str title: The message title.

        :param str message: The actual message.

        :param str icon: The icon path.
        """
        ...

    def hide(self) -> None:
        """Hides the notification displayed by :meth:`notify`."""
        ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gtk
from nbxmpp.modules.vcard4 import VCard
from nbxmpp.protocol import Iq

from gajim.gtk.vcard_grid import VCardGrid
from gajim.gtk.window import GajimAppWindow

from . import util

VCARD_NODE = """
<vcard xmlns="urn:ietf:params:xml:ns:vcard-4.0">
    <fn><text>Peter Saint-Andre</text></fn>
    <n><surname>Saint-Andre</surname><given>Peter</given><additional></additional></n>
    <nickname><text>stpeter</text></nickname>
    <nickname><text>psa</text></nickname>
    <photo><uri>https://stpeter.im/images/stpeter_oscon.jpg</uri></photo>
    <bday><date>1966-08-06</date></bday>
    <adr>
        <parameters>
        <type><text>work</text><text>voice</text></type>
        <pref><integer>1</integer></pref>
        </parameters>
        <ext>Suite 600</ext>
        <street>1899 Wynkoop Street</street>
        <locality>Denver</locality>
        <region>CO</region>
        <code>80202</code>
        <country>USA</country>
    </adr>
    <adr>
        <parameters><type><text>home</text></type></parameters>
        <ext></ext>
        <street></street>
        <locality>Parker</locality>
        <region>CO</region>
        <code>80138</code>
        <country>USA</country>
    </adr>
    <tel>
        <parameters>
        <type><text>work</text><text>voice</text></type>
        <pref><integer>1</integer></pref>
        </parameters>
        <uri>tel:+1-303-308-3282</uri>
    </tel>
    <tel>
        <parameters><type><text>work</text><text>fax</text></type></parameters>
        <uri>tel:+1-303-308-3219</uri>
    </tel>
    <tel>
        <parameters>
        <type><text>cell</text><text>voice</text><text>text</text></type>
        </parameters>
        <uri>tel:+1-720-256-6756</uri>
    </tel>
    <tel>
        <parameters><type><text>home</text><text>voice</text></type></parameters>
        <uri>tel:+1-303-555-1212</uri>
    </tel>
    <geo><uri>geo:39.59,-105.01</uri></geo>
    <title><text>Executive Director</text></title>
    <role><text>Patron Saint</text></role>
    <org>
        <parameters><type><text>work</text></type></parameters>
        <text>XMPP Standards Foundation</text>
    </org>
    <url><uri>https://stpeter.im/</uri></url>
    <note>
        <text>More information about me is located on my personal website: https://stpeter.im/</text>
    </note>
    <gender><sex><text>M</text></sex></gender>
    <lang>
        <parameters><pref>1</pref></parameters>
        <language-tag>en</language-tag>
    </lang>
    <email>
        <parameters><type><text>work</text></type></parameters>
        <text>psaintan@cisco.com</text>
    </email>
    <email>
        <parameters><type><text>home</text></type></parameters>
        <text>stpeter@jabber.org</text>
    </email>
    <impp>
        <parameters><type><text>work</text></type></parameters>
        <uri>xmpp:psaintan@cisco.com</uri>
    </impp>
    <impp>
        <parameters><type><text>home</text></type></parameters>
        <uri>xmpp:stpeter@jabber.org</uri>
    </impp>
    <key>
        <uri>https://stpeter.im/stpeter.asc</uri>
    </key>
</vcard>
"""

VCARD = VCard.from_node(Iq(node=VCARD_NODE))  # type: ignore
ACCOUNT = "test"


class TestVCardGrid(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=800,
            default_height=700,
            add_window_padding=True,
            header_bar=True,
        )

        self._is_editable = False

        main_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=18)
        main_box.set_valign(Gtk.Align.FILL)
        self.set_child(main_box)

        scrolled_window = Gtk.ScrolledWindow(hexpand=True, vexpand=True)
        main_box.append(scrolled_window)

        buttons_box = Gtk.Box(halign=Gtk.Align.CENTER)
        main_box.append(buttons_box)

        edit_toggle = Gtk.Button(label="Toggle Edit")
        edit_toggle.connect("clicked", self._on_edit_toggled)
        buttons_box.append(edit_toggle)

        self._vcard_grid = VCardGrid(ACCOUNT)
        self._vcard_grid.set_hexpand(True)
        self._vcard_grid.set_vcard(VCARD)
        scrolled_window.set_child(self._vcard_grid)

    def _on_edit_toggled(self, _button: Gtk.Button) -> None:
        self._vcard_grid.set_editable(not self._is_editable)
        self._is_editable = not self._is_editable

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestVCardGrid()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gio
from gi.repository import GObject
from gi.repository import Gtk

from gajim.gtk.window import GajimAppWindow

from . import util


class Country(GObject.Object):
    __gtype_name__ = "Country"

    country_id = GObject.Property(type=str)
    country_name = GObject.Property(type=str)
    country_pm = GObject.Property(type=str)


class TestListView(GajimAppWindow):
    def __init__(self) -> None:
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=800,
            default_height=800,
            add_window_padding=True,
            header_bar=True,
        )

        nodes = {
            "at": ("Austria", "Van der Bellen"),
            "uk": ("United Kingdom", "Charles III"),
            "us": ("United States", "Biden"),
        }

        self.model = Gio.ListStore(item_type=Country)
        for n in nodes:
            self.model.append(
                Country(country_id=n, country_name=nodes[n][0], country_pm=nodes[n][1])
            )

        factory = Gtk.SignalListItemFactory()
        factory.connect("setup", self._on_factory_setup)
        factory.connect("bind", self._on_factory_bind, "country_name")
        factory.connect("unbind", self._on_factory_unbind, "country_name")
        factory.connect("teardown", self._on_factory_teardown)

        self.lv = Gtk.ListView(
            model=Gtk.NoSelection(model=self.model),
            factory=factory,
            hexpand=True,
        )

        box = Gtk.Box(
            orientation=Gtk.Orientation.VERTICAL, spacing=12, valign=Gtk.Align.CENTER
        )
        box.props.margin_start = 12
        box.props.margin_end = 12
        box.props.margin_top = 6
        box.props.margin_bottom = 6
        box.append(Gtk.Label(label="Some Table:"))
        box.append(self.lv)

        self.set_child(box)

    def _cleanup(self) -> None:
        pass

    def _on_factory_setup(
        self, factory: Gtk.SignalListItemFactory, list_item: Gtk.ListItem
    ) -> None:
        cell = Gtk.Inscription()
        cell._binding = None  # pyright: ignore
        list_item.set_child(cell)

    def _on_factory_bind(
        self, factory: Gtk.SignalListItemFactory, list_item: Gtk.ListItem, what: str
    ) -> None:
        country = list_item.get_item()
        cell = list_item.get_child()
        assert cell is not None
        cell._binding = country.bind_property(  # pyright: ignore
            what, cell, "text", GObject.BindingFlags.SYNC_CREATE
        )

    def _on_factory_unbind(
        self, factory: Gtk.SignalListItemFactory, list_item: Gtk.ListItem, what: str
    ) -> None:
        cell = list_item.get_child()
        if cell._binding:  # pyright: ignore
            cell._binding.unbind()  # pyright: ignore
            cell._binding = None  # pyright: ignore

    def _on_factory_teardown(
        self, factory: Gtk.SignalListItemFactory, list_item: Gtk.ListItem
    ) -> None:
        pass


util.init_settings()

window = TestListView()
window.show()

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

from typing import Any

from unittest.mock import MagicMock

from gi.repository import Gdk
from gi.repository import GLib
from gi.repository import Gtk
from nbxmpp import Iq
from nbxmpp import JID
from nbxmpp.const import InviteType
from nbxmpp.structs import BookmarkData
from nbxmpp.structs import DiscoInfo

from gajim.common import app
from gajim.common import events
from gajim.common.const import AvatarSize
from gajim.common.const import CSSPriority
from gajim.common.helpers import Observable
from gajim.common.modules.contacts import BareContact
from gajim.common.modules.contacts import ContactSettings
from gajim.common.modules.contacts import GroupchatContact
from gajim.common.modules.contacts import GroupChatSettings

from gajim.gtk.activity_list import ActivityListView
from gajim.gtk.activity_page import ActivityPage
from gajim.gtk.app_side_bar import AppSideBar
from gajim.gtk.avatar import AvatarStorage
from gajim.gtk.avatar import generate_default_avatar
from gajim.gtk.chat_page import ChatPage
from gajim.gtk.util.misc import convert_surface_to_texture
from gajim.gtk.window import GajimAppWindow

from . import util

ACCOUNT = "testacc1"
FROM_JID = JID.from_string("contact@example.org")
FROM_JID_2 = JID.from_string("contact2@example.org")
MUC_JID = JID.from_string("test@conference.example.org")
MUC_JID_2 = JID.from_string("test2@conference.example.org")


class TestActivityFeed(GajimAppWindow):
    def __init__(self) -> None:
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=1000,
            default_height=800,
            header_bar=True,
        )

        app.app = MagicMock()
        app.app.avatar_storage.get_gajim_circle_icon.side_effect = (
            self._get_gajim_circle_icon
        )

        chat_page = ChatPage()

        # activity_list.connect("activate", self._on_activity_row_activate)
        # activity_list.connect("unselected", self._on_activity_item_unselected)

        # Replace ChatPage's internal activity list with test object
        # to track unread count in sidebar
        # chat_page._activity_list = activity_list

        self._activity_page = ActivityPage()

        self._app_sidebar = AppSideBar()
        self._app_sidebar.set_chat_page(chat_page)
        self._app_sidebar.set_valign(Gtk.Align.START)

        update_button = Gtk.Button.new_with_label("Updates")
        update_button.connect("clicked", self._on_update_button_clicked)

        subscription_button = Gtk.Button.new_with_label("Subscription")
        subscription_button.connect("clicked", self._on_subscription_button_clicked)

        invitation_button = Gtk.Button.new_with_label("Invitation")
        invitation_button.connect("clicked", self._on_invitation_button_clicked)

        button_box = Gtk.Box(
            spacing=12,
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.END,
            hexpand=True,
            margin_bottom=12,
        )
        button_box.add_css_class("toolbar")
        button_box.add_css_class("osd")
        button_box.append(update_button)
        button_box.append(subscription_button)
        button_box.append(invitation_button)

        overlay = Gtk.Overlay(hexpand=True)
        overlay.set_child(chat_page)
        overlay.add_overlay(button_box)
        self.set_child(overlay)

    def _cleanup(self) -> None:
        pass

    def _get_gajim_circle_icon(self, size: int, scale: int) -> Gdk.Texture | None:
        return AvatarStorage.get_gajim_circle_icon(size, scale)

    def show_activity_page(self) -> None:
        self._app_sidebar.show_activity_page()
        self._activity_page.show_default_page()

    def _on_activity_row_activate(
        self, listview: ActivityListView, position: int
    ) -> None:
        item = listview.get_listitem(position)
        self._activity_page.process_row_activated(item)

    def _on_activity_item_unselected(self, listview: ActivityListView) -> None:
        self._activity_page.show_default_page()

    def _on_activity_row_removed(self, _listview: ActivityListView) -> None:
        self._activity_page.show_default_page()

    def _on_update_button_clicked(self, _button: Gtk.Button) -> None:
        app.plugin_repository.trigger_plugin_update()  # type: ignore
        app.ged.raise_event(events.AllowGajimUpdateCheck())
        app.ged.raise_event(
            events.GajimUpdateAvailable("2.0.0", "https://gajim.org/download")
        )

        # Trigger plugin update successful event with delay, because it removes
        # available update message
        GLib.timeout_add_seconds(
            10,
            app.plugin_repository.trigger_plugin_update_successful,  # type: ignore
        )

    def _on_subscription_button_clicked(self, _button: Gtk.Button) -> None:
        event = events.SubscribePresenceReceived(
            account=ACCOUNT,
            jid=FROM_JID,
            status="",
            user_nick="User Nick",
            is_transport=False,
        )
        app.ged.raise_event(event)

        event = events.UnsubscribedPresenceReceived(
            account=ACCOUNT,
            jid=FROM_JID_2,
        )
        app.ged.raise_event(event)

    def _on_invitation_button_clicked(self, _button: Gtk.Button) -> None:
        muc_info = DiscoInfo(
            Iq(frm=JID.from_string("test@example.org")), [], [], [], None
        )
        event = events.MucInvitation(
            ACCOUNT,
            info=muc_info,
            muc=MUC_JID,
            from_=FROM_JID,
            reason=None,
            password=None,
            type=InviteType.DIRECT,
            continued=False,
            thread=None,
        )
        app.ged.raise_event(event)

        event = events.MucDecline(
            ACCOUNT,
            muc=MUC_JID_2,
            from_=FROM_JID_2,
            reason=None,
        )
        app.ged.raise_event(event)


class TestPluginRepository(Observable):
    def __init__(self):
        super().__init__()

    def trigger_plugin_update(self) -> None:
        self.notify("plugin-updates-available", [])

    def trigger_plugin_update_successful(self) -> None:
        self.notify("auto-update-finished")


class TestModule(MagicMock):
    def __init__(self) -> None:  # pyright: ignore
        super().__init__()

    def get_contact(self, jid: JID, groupchat: bool = False) -> MagicMock:
        if jid == MUC_JID:
            return self._get_contact(jid, groupchat=True)

        return self._get_contact(jid, groupchat=groupchat)

    def _get_contact(self, jid: JID, groupchat: bool = False) -> MagicMock:
        if groupchat:
            contact = MagicMock(spec=GroupchatContact)
            contact.settings = GroupChatSettings(ACCOUNT, jid)
        else:
            contact = MagicMock(spec=BareContact)
            contact.settings = ContactSettings(ACCOUNT, jid)

        contact.is_groupchat = groupchat

        contact.connect = MagicMock()
        contact.account = ACCOUNT
        contact.jid = jid
        contact.name = "Test Contact"
        avatar = convert_surface_to_texture(
            generate_default_avatar("T", (0.2, 0.1, 0.7), AvatarSize.ACTIVITY_PAGE, 1)
        )
        contact.get_avatar = MagicMock(return_value=avatar)
        return contact

    def get_name_from_bookmark(self, jid: JID) -> str:
        return "Group Chat Name"

    def get_bookmark(self, room_jid: JID) -> BookmarkData:
        return BookmarkData(MUC_JID, "Group Chat Name", "Nick")

    def decline(self, *args: Any) -> None:
        pass


class TestClient(MagicMock):
    def __init__(self) -> None:  # pyright: ignore
        super().__init__()

    def get_module(self) -> TestModule:
        return TestModule()


app.plugin_repository = TestPluginRepository()

util.init_settings()
app.settings.add_account("testacc1")
app.settings.set_account_setting("testacc1", "active", True)
app.settings.add_account("testacc2")
app.settings.set_account_setting("testacc2", "active", True)

app.window = MagicMock()
app.commands = MagicMock()
app.plugin_manager = MagicMock()

app.get_client = MagicMock(return_value=TestClient)

css = ".gajim_class_1 { background-color: purple; }"
provider = Gtk.CssProvider()
provider.load_from_bytes(GLib.Bytes.new(css.encode()))

display = Gdk.Display.get_default()
assert display is not None
Gtk.StyleContext.add_provider_for_display(display, provider, CSSPriority.APPLICATION)

app.css_config = MagicMock()
app.css_config.get_dynamic_class = MagicMock(return_value="gajim_class_1")

window = TestActivityFeed()
window.show()

app.window.show_activity_page = window.show_activity_page

util.run_app()



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gi.repository import Gdk
from gi.repository import Graphene
from gi.repository import Gsk
from gi.repository import Gtk

from gajim.gtk.window import GajimAppWindow

from . import util

DEFAULT_IMAGE_FILE_PATH = (
    util.get_gajim_dir() / "data/icons/hicolor/96x96/apps/gajim.png"
)


class TestSnapshot(GajimAppWindow):
    def __init__(self):
        GajimAppWindow.__init__(
            self,
            name="",
            title=__class__.__name__,
            default_width=600,
            default_height=600,
            add_window_padding=True,
            header_bar=True,
        )

        box = Gtk.Box(
            halign=Gtk.Align.CENTER,
            valign=Gtk.Align.CENTER,
            hexpand=True,
            orientation=Gtk.Orientation.VERTICAL,
            spacing=12,
        )
        self.set_child(box)

        self._image = Gtk.Image()
        self._image.set_pixel_size(100)
        box.append(self._image)

        self._paint()

    def _paint(self) -> None:
        texture = Gdk.Texture.new_from_filename(str(DEFAULT_IMAGE_FILE_PATH))

        snapshot = Gtk.Snapshot.new()
        rect = Graphene.Rect().init(0, 0, texture.get_width(), texture.get_height())
        snapshot.append_texture(texture, rect)

        cutout_rect = Graphene.Rect().init(50, 50, 50, 50)

        color = Gdk.RGBA()
        color.parse("rgb(70, 0, 190)")

        rounded_rect = Gsk.RoundedRect()
        rounded_rect.init_from_rect(cutout_rect, radius=25)

        snapshot.push_rounded_clip(rounded_rect)
        snapshot.append_color(color, cutout_rect)
        snapshot.pop()

        overlay_rect = Graphene.Rect().init(55, 55, 40, 40)
        snapshot.append_scaled_texture(
            texture, Gsk.ScalingFilter.TRILINEAR, overlay_rect
        )

        self._image.set_from_paintable(snapshot.to_paintable())

    def _cleanup(self) -> None:
        pass


util.init_settings()

window = TestSnapshot()
window.show()

util.run_app()



import typing

from PIL import Image

"""
This type stub file was generated by pyright.
"""

class Icon:
    """A representation of a system tray icon.

    The icon is initially hidden. Set :attr:`visible` to ``True`` to show it.

    :param str name: The name of the icon. This is used by the system to
        identify the icon.

    :param icon: The icon to use. If this is specified, it must be a
        :class:`PIL.Image.Image` instance.

    :param str title: A short title for the icon.

    :param menu: A menu to use as popup menu. This can be either an instance of
        :class:`Menu` or an iterable, which will be interpreted as arguments to
        the :class:`Menu` constructor, or ``None``, which disables the menu.

        The behaviour of the menu depends on the platform. Only one action is
        guaranteed to be invocable: the first menu item whose
        :attr:`~pystray.MenuItem.default` attribute is set.

        Some platforms allow both menu interaction and a special way of
        activating the default action, some platform allow only either an
        invisible menu with a default entry as special action or a full menu
        with no special way to activate the default item, and some platforms do
        not support a menu at all.

    :param kwargs: Any non-standard platform dependent options. These should be
        prefixed with the platform name thus: ``appindicator_``, ``darwin_``,
        ``gtk_``, ``win32_`` or ``xorg_``.

        Supported values are:

        ``darwin_nsapplication``
            An ``NSApplication`` instance used to run the event loop. If this
            is not specified, the shared application will be used.
    """

    HAS_DEFAULT_ACTION: bool = ...
    HAS_MENU: bool = ...
    HAS_MENU_RADIO: bool = ...
    HAS_NOTIFICATION: bool = ...
    SETUP_THREAD_TIMEOUT: float = ...
    def __init__(
        self,
        name: str,
        icon: "Image.Image" = ...,
        title: str = ...,
        menu: Menu | None = ...,
        **kwargs: typing.Any,
    ) -> None: ...
    def __del__(self) -> None: ...
    def __call__(self) -> None: ...
    @property
    def name(self) -> str:
        """The name passed to the constructor."""
        ...

    @property
    def icon(self) -> Image.Image | None:
        """The current icon.

        Setting this to a falsy value will hide the icon. Setting this to an
        image while the icon is hidden has no effect until the icon is shown.
        """
        ...

    @icon.setter
    def icon(self, value: Image.Image | None) -> None: ...
    @property
    def title(self) -> str:
        """The current icon title."""
        ...

    @title.setter
    def title(self, value: str) -> None: ...
    @property
    def menu(self) -> Menu | None:
        """The menu.

        Setting this to a falsy value will disable the menu.
        """
        ...

    @menu.setter
    def menu(self, value: Menu | None) -> None: ...
    @property
    def visible(self) -> bool:
        """Whether the icon is currently visible.

        :raises ValueError: if set to ``True`` and no icon image has been set
        """
        ...

    @visible.setter
    def visible(self, value: bool) -> None: ...
    def run(self, setup: typing.Any = ...) -> None:
        """Enters the loop handling events for the icon.

        This method is blocking until :meth:`stop` is called. It *must* be
        called from the main thread.

        :param callable setup: An optional callback to execute in a separate
            thread once the loop has started. It is passed the icon as its sole
            argument.

            Please note that this function is started in a thread, and when the
            icon is stopped, an attempt to join this thread is made, so
            stopping the icon may be blocking for up to
            ``SETUP_THREAD_TIMEOUT`` seconds if the function is not
            well-behaved.

            If not specified, a simple setup function setting :attr:`visible`
            to ``True`` is used. If you specify a custom setup function, you
            must explicitly set this attribute.
        """
        ...

    def run_detached(self, setup: typing.Any = ...) -> None:
        """Prepares for running the loop handling events detached.

        This allows integrating *pystray* with other libraries requiring a
        mainloop. Call this method before entering the mainloop of the other
        library.

        Depending on the backend used, calling this method may require special
        preparations:

        macOS
            Pass an instance of ``NSApplication`` retrieved from the library
            with which you are integrating as the argument
            ``darwin_nsapplication``. This will allow this library to integrate
            with the main loop.

        :param callable setup: An optional callback to execute in a separate
            thread once the loop has started. It is passed the icon as its sole
            argument.

            If not specified, a simple setup function setting :attr:`visible`
            to ``True`` is used. If you specify a custom setup function, you
            must explicitly set this attribute.

        :raises NotImplementedError: if this is not implemented for the
            preparations taken
        """
        ...

    def stop(self) -> None:
        """Stops the loop handling events for the icon.

        If the icon is not running, calling this method has no effect.
        """
        ...

    def update_menu(self) -> None:
        """Updates the menu.

        If the properties of the menu descriptor are dynamic, that is, any are
        defined by callables and not constants, and the return values of these
        callables change by actions other than the menu item activation
        callbacks, calling this function is required to keep the menu in sync.

        This is required since not all supported platforms allow the menu to be
        generated when shown.

        For simple use cases where menu changes are triggered by interaction
        with the menu, this method is not necessary.
        """
        ...

    def notify(self, message: str, title: str | None = ...) -> None:
        """Displays a notification.

        The notification will generally be visible until
        :meth:`remove_notification` is called.

        The class field :attr:`HAS_NOTIFICATION` indicates whether this feature
        is supported on the current platform.

        :param str message: The message of the notification.

        :param str title: The title of the notification. This will be replaced
            with :attr:`title` if ``None``.
        """
        ...

    def remove_notification(self) -> None:
        """Remove a notification."""
        ...

class MenuItem:
    """A single menu item.

    A menu item is immutable.

    It has a text and an action. The action is either a callable of a menu. It
    is callable; when called, the activation callback is called.

    The :attr:`visible` attribute is provided to make menu creation easier; all
    menu items with this value set to ``False`` will be discarded when a
    :class:`Menu` is constructed.
    """

    def __init__(
        self,
        text: str,
        action: typing.Any,
        checked: typing.Any = ...,
        radio: bool = ...,
        default: bool = ...,
        visible: bool = ...,
        enabled: bool = ...,
    ) -> None: ...
    def __call__(self, icon: Icon) -> typing.Any | None: ...
    def __str__(self) -> str: ...
    @property
    def text(self) -> str:
        """The menu item text."""
        ...

    @property
    def checked(self) -> bool | None:
        """Whether this item is checked.

        This can be either ``True``, which implies that the item is checkable
        and checked, ``False``, which implies that the item is checkable but
        not checked, and ``None`` for uncheckable items.

        Depending on platform, uncheckable items may be rendered differently
        from unchecked items.
        """
        ...

    @property
    def radio(self) -> bool | typing.Literal[False]:
        """Whether this item is a radio button.

        This is only used for checkable items. It is always set to ``False``
        for uncheckable items.
        """
        ...

    @property
    def default(self) -> bool:
        """Whether this is the default menu item."""
        ...

    @property
    def visible(self) -> bool:
        """Whether this menu item is visible.

        If the action for this menu item is a menu, that also has to be visible
        for this property to be ``True``.
        """
        ...

    @property
    def enabled(self) -> bool:
        """Whether this menu item is enabled."""
        ...

    @property
    def submenu(self) -> Menu | None:
        """The submenu used by this menu item, or ``None``."""
        ...

class Menu:
    """A description of a menu.

    A menu description is immutable.

    It is created with a sequence of :class:`Menu.Item` instances, or a single
    callable which must return a generator for the menu items.

    First, non-visible menu items are removed from the list, then any instances
    of :attr:`SEPARATOR` occurring at the head or tail of the item list are
    removed, and any consecutive separators are reduced to one.
    """

    SEPARATOR: MenuItem = ...

    def __init__(self, *items: typing.Any) -> None: ...
    @property
    def items(self) -> tuple[MenuItem, ...] | tuple[typing.Any]:
        """All menu items."""
        ...

    @property
    def visible(self) -> bool:
        """Whether this menu is visible."""
        ...

    def __call__(self, icon: Icon) -> None: ...
    def __iter__(self) -> typing.Iterator[typing.Any]: ...
    def __bool__(self) -> bool: ...
    __nonzero__: typing.Any = ...

    def __str__(self) -> str: ...



# This file is part of Gajim.
#
# SPDX-License-Identifier: GPL-3.0-or-later

from functools import partial
from unittest.mock import MagicMock

from gi.repository import Gtk
from nbxmpp.modules.dataforms import create_field
from nbxmpp.modules.dataforms import SimpleDataForm

from gajim.common import app

from gajim.gtk.change_password import ChangePassword

from . import util

app.get_client = MagicMock()

fields = [
    create_field(typ="text-single", label="Username", var="username"),
    create_field(typ="text-single", label="Old Password", var="old_password"),
    create_field(typ="text-single", label="Mothers name", var="mother", required=True),
]

form = SimpleDataForm(type_="form", fields=fields)


def _apply(self: ChangePassword, next_stage: bool = False) -> None:
    if next_stage:
        self.get_page("next_stage").get_submit_form()
    else:
        self.get_page("next_stage").set_form(form)
        self.show_page("next_stage", Gtk.StackTransitionType.SLIDE_LEFT)


util.init_settings()

window = ChangePassword("")
window._on_apply = partial(_apply, window)  # type: ignore
window.show()

util.run_app()
